@@ -1,6 +1,7 @@
 # coding: utf-8
 #
-#  Copyright (c) 2008—2014 Andy Mikhailenko
+#  Copyright (c) 2018-present Justin Mayer
+#  Copyright (c) 2008—2016 Andy Mikhailenko
 #
 #  This file is part of django-autoslug.
 #
@@ -10,7 +11,9 @@
 #
 
 # django
+from django.conf import settings
 from django.db.models.fields import SlugField
+from django.db.models.signals import post_save
 
 # 3rd-party
 try:
@@ -18,19 +21,23 @@ try:
 except ImportError:
     introspector = lambda self: [], {}
 
-# this app
-from autoslug.settings import slugify
-from autoslug import utils
+try:
+    from modeltranslation import utils as modeltranslation_utils
+except ImportError:
+    modeltranslation_utils = None
 
+# this app
+from autoslug.settings import slugify, autoslug_modeltranslation_enable
+from autoslug import utils
 
 __all__ = ['AutoSlugField']
 
-SLUG_INDEX_SEPARATOR = '-'    # the "-" in "foo-2"
+SLUG_INDEX_SEPARATOR = '-'  # the "-" in "foo-2"
 
-try:                 # pragma: nocover
+try:  # pragma: nocover
     # Python 2.x
     basestring
-except NameError:    # pragma: nocover
+except NameError:  # pragma: nocover
     # Python 3.x
     basestring = str
 
@@ -87,7 +94,9 @@ class AutoSlugField(SlugField):
         `unique_with`). The reasoning is that autosaved dates and other such
         fields must be already processed before using them in the AutoSlugField.
 
-    Example usage::
+    Example usage:
+
+    .. code-block:: python
 
         from django.db import models
         from autoslug import AutoSlugField
@@ -102,7 +111,9 @@ class AutoSlugField(SlugField):
             slug = AutoSlugField(populate_from='title', unique_with='pub_date__month')
 
 
-    More options::
+    More options:
+
+    .. code-block:: python
 
         # slugify but allow non-unique slugs
         slug = AutoSlugField()
@@ -122,11 +133,11 @@ class AutoSlugField(SlugField):
         # ensure that slug is unique with given date AND category
         slug = AutoSlugField(unique_with=('pub_date','category'))
 
-        # ensure that slug in unique with an external object
+        # ensure that slug is unique with an external object
         # assuming that author=ForeignKey(Author)
         slug = AutoSlugField(unique_with='author')
 
-        # ensure that slug in unique with a subset of external objects (by lookups)
+        # ensure that slug is unique with a subset of external objects (by lookups)
         # assuming that author=ForeignKey(Author)
         slug = AutoSlugField(unique_with='author__name')
 
@@ -162,6 +173,7 @@ class AutoSlugField(SlugField):
         def custom_slugify(value):
             return default_slugify(value).replace('-', '_')
         slug = AutoSlugField(slugify=custom_slugify)
+
     """
     def __init__(self, *args, **kwargs):
         kwargs['max_length'] = kwargs.get('max_length', 50)
@@ -189,19 +201,66 @@ class AutoSlugField(SlugField):
         if 'db_index' not in kwargs:
             kwargs['db_index'] = True
 
+        # A boolean instructing the field to accept Unicode letters in
+        # addition to ASCII letters. Defaults to False.
+        self.allow_unicode = kwargs.pop('allow_unicode', False)
+
         # When using model inheritence, set manager to search for matching
         # slug values
         self.manager = kwargs.pop('manager', None)
+        self.manager_name = kwargs.pop('manager_name', None)
 
         self.always_update = kwargs.pop('always_update', False)
         super(SlugField, self).__init__(*args, **kwargs)
 
+    def deconstruct(self):
+        name, path, args, kwargs = super(AutoSlugField, self).deconstruct()
+
+        if self.max_length == 50:
+            kwargs.pop('max_length', None)
+
+        if self.populate_from is not None:
+            kwargs['populate_from'] = self.populate_from
+            if self.editable is not False:
+                kwargs['editable'] = self.editable
+
+        if self.unique_with != ():
+            kwargs['unique_with'] = self.unique_with
+            kwargs.pop('unique', None)
+
+        if self.slugify != slugify:
+            kwargs['slugify'] = self.slugify
+
+        if self.index_sep != SLUG_INDEX_SEPARATOR:
+            kwargs['sep'] = self.index_sep
+
+        kwargs.pop('db_index', None)
+
+        if self.manager is not None:
+            kwargs['manager'] = self.manager
+
+        if self.manager_name is not None:
+            kwargs['manager_name'] = self.manager_name
+
+        if self.always_update:
+            kwargs['always_update'] = self.always_update
+
+        if 'manager' in kwargs:
+            del kwargs['manager']
+
+        return name, path, args, kwargs
+
     def pre_save(self, instance, add):
 
         # get currently entered slug
         value = self.value_from_object(instance)
 
-        manager = self.manager
+        if self.manager is not None:
+            manager = self.manager
+        elif self.manager_name is not None:
+            manager = getattr(self.model, self.manager_name)
+        else:
+            manager = None
 
         # autopopulate
         if self.always_update or (self.populate_from and not value):
@@ -210,21 +269,19 @@ class AutoSlugField(SlugField):
             # pragma: nocover
             if __debug__ and not value and not self.blank:
                 print('Failed to populate slug %s.%s from %s' % \
-                    (instance._meta.object_name, self.name, self.populate_from))
+                      (instance._meta.object_name, self.name, self.populate_from))
 
+        slug = None
         if value:
             slug = self.slugify(value)
-        else:
+        if not slug:
             slug = None
 
             if not self.blank:
-                slug = instance._meta.module_name
+                slug = instance._meta.model_name
             elif not self.null:
                 slug = ''
 
-        if not self.blank:
-            assert slug, 'slug is defined before trying to ensure uniqueness'
-
         if slug:
             slug = utils.crop_slug(self, slug)
 
@@ -237,8 +294,15 @@ class AutoSlugField(SlugField):
         # make the updated slug available as instance attribute
         setattr(instance, self.name, slug)
 
+        # modeltranslation support
+        if 'modeltranslation' in settings.INSTALLED_APPS \
+                and not hasattr(self.populate_from, '__call__') \
+                and autoslug_modeltranslation_enable:
+            post_save.connect(modeltranslation_update_slugs, sender=type(instance))
+
         return slug
 
+
     def south_field_triple(self):
         "Returns a suitable description of this field for South."
         args, kwargs = introspector(self)
@@ -247,3 +311,44 @@ class AutoSlugField(SlugField):
             'unique_with': repr(self.unique_with)
         })
         return ('autoslug.fields.AutoSlugField', args, kwargs)
+
+
+def modeltranslation_update_slugs(sender, **kwargs):
+    # https://bitbucket.org/neithere/django-autoslug/pull-request/11/modeltranslation-support-fix-issue-19/
+    # http://django-modeltranslation.readthedocs.org
+    #
+    # TODO: tests
+    #
+    if not modeltranslation_utils:
+        return
+
+    instance = kwargs['instance']
+    slugs = {}
+
+    for field in instance._meta.fields:
+        if type(field) != AutoSlugField:
+            continue
+        if not field.populate_from:
+            continue
+        for lang in settings.LANGUAGES:
+            lang_code, _ = lang
+            lang_code = lang_code.replace('-', '_')
+
+            populate_from_localized = modeltranslation_utils.build_localized_fieldname(field.populate_from, lang_code)
+            field_name_localized = modeltranslation_utils.build_localized_fieldname(field.name, lang_code)
+
+            # The source field or the slug field itself may not be registered
+            # with translator
+            if not hasattr(instance, populate_from_localized):
+                continue
+            if not hasattr(instance, field_name_localized):
+                continue
+
+            populate_from_value = getattr(instance, populate_from_localized)
+            field_value = getattr(instance, field_name_localized)
+
+            if not field_value or field.always_update:
+                slug = field.slugify(populate_from_value)
+                slugs[field_name_localized] = slug
+
+    sender.objects.filter(pk=instance.pk).update(**slugs)
