@@ -1,6 +1,7 @@
 # coding: utf-8
 #
-#  Copyright (c) 2008—2014 Andy Mikhailenko
+#  Copyright (c) 2018-present Justin Mayer
+#  Copyright (c) 2008—2016 Andy Mikhailenko
 #
 #  This file is part of django-autoslug.
 #
@@ -49,10 +50,25 @@ Django settings that affect django-autoslug:
 .. _pytils: http://pypi.python.org/pypi/pytils
 .. _translitcodec: http://pypi.python.org/pypi/translitcodec
 
+`AUTOSLUG_MODELTRANSLATION_ENABLE`
+  Django-autoslug support of modeltranslation_ is still experimental.
+  If you wish to enable it, please set this option to `True` in your project
+  settings.  Default is `False`.
+
+.. _modeltranslation: http://django-modeltranslation.readthedocs.org
+
 """
 from django.conf import settings
-from django.core.urlresolvers import get_callable
+from django import VERSION
+
+if VERSION >= (2, 0):
+    from django.urls import get_callable
+else:
+    from django.core.urlresolvers import get_callable
 
 # use custom slugifying function if any
 slugify_function_path = getattr(settings, 'AUTOSLUG_SLUGIFY_FUNCTION', 'autoslug.utils.slugify')
 slugify = get_callable(slugify_function_path)
+
+# enable/disable modeltranslation support
+autoslug_modeltranslation_enable = getattr(settings, 'AUTOSLUG_MODELTRANSLATION_ENABLE', False)
